@@ -2,11 +2,10 @@
 Patchify.py
 """
 
-from itertools import product
 from typing import Tuple, Union, cast
 
 import numpy as np
-from skimage.util import view_as_windows
+from .view_as_windows import view_as_windows
 
 
 Imsize = Union[Tuple[int, int], Tuple[int, int, int]]
@@ -78,20 +77,17 @@ def _unpatchify2d(  # pylint: disable=too-many-locals
 
     n_h, n_w, p_h, p_w = patches.shape
 
-    # Calculat the overlap size in each axis
-    o_w = 0 if n_w == 1 else (n_w * p_w - i_w) / (n_w - 1)
-    o_h = 0 if n_h == 1 else (n_h * p_h - i_h) / (n_h - 1)
+    s_w = 0 if n_w <= 1 else (i_w - p_w) / (n_w - 1)
+    s_h = 0 if n_h <= 1 else (i_h - p_h) / (n_h - 1)
 
-    # The overlap should be integer, otherwise the patches are unable
-    # to reconstruct into a image with given shape
-    assert int(o_w) == o_w
-    assert int(o_h) == o_h
-
-    o_w = int(o_w)  # overlap
-    o_h = int(o_h)
-
-    s_w = p_w - o_w  # step
-    s_h = p_h - o_h
+    # The step size should be same for all patches, otherwise the patches are unable
+    # to reconstruct into a image
+    if int(s_w) != s_w:
+        raise NonUniformStepSizeError(i_w, n_w, p_w, s_w)
+    if int(s_h) != s_h:
+        raise NonUniformStepSizeError(i_h, n_h, p_h, s_h)
+    s_w = int(s_w)
+    s_h = int(s_h)
 
     i, j = 0, 0
 
@@ -126,24 +122,22 @@ def _unpatchify3d(  # pylint: disable=too-many-locals
 
     n_h, n_w, n_c, p_h, p_w, p_c = patches.shape
 
-    # Calculat the overlap size in each axis
-    o_w = 0 if n_w == 1 else (n_w * p_w - i_w) / (n_w - 1)
-    o_h = 0 if n_h == 1 else (n_h * p_h - i_h) / (n_h - 1)
-    o_c = 0 if n_c == 1 else (n_c * p_c - i_c) / (n_c - 1)
+    s_w = 0 if n_w <= 1 else (i_w - p_w) / (n_w - 1)
+    s_h = 0 if n_h <= 1 else (i_h - p_h) / (n_h - 1)
+    s_c = 0 if n_c <= 1 else (i_c - p_c) / (n_c - 1)
 
-    # The overlap should be integer, otherwise the patches are unable
-    # to reconstruct into a image with given shape
-    assert int(o_w) == o_w
-    assert int(o_h) == o_h
-    assert int(o_c) == o_c
+    # The step size should be same for all patches, otherwise the patches are unable
+    # to reconstruct into a image
+    if int(s_w) != s_w:
+        raise NonUniformStepSizeError(i_w, n_w, p_w, s_w)
+    if int(s_h) != s_h:
+        raise NonUniformStepSizeError(i_h, n_h, p_h, s_h)
+    if int(s_c) != s_c:
+        raise NonUniformStepSizeError(i_c, n_c, p_c, s_c)
 
-    o_w = int(o_w)
-    o_h = int(o_h)
-    o_c = int(o_c)
-
-    s_w = p_w - o_w
-    s_h = p_h - o_h
-    s_c = p_c - o_c
+    s_w = int(s_w)
+    s_h = int(s_h)
+    s_c = int(s_c)
 
     i, j, k = 0, 0, 0
 
@@ -169,3 +163,21 @@ def _unpatchify3d(  # pylint: disable=too-many-locals
             raise RuntimeError("Unreachable")
 
     return image
+
+
+class NonUniformStepSizeError(RuntimeError):
+    def __init__(
+        self, imsize: int, n_patches: int, patch_size: int, step_size: float
+    ) -> None:
+        super().__init__(imsize, n_patches, patch_size, step_size)
+        self.n_patches = n_patches
+        self.patch_size = patch_size
+        self.imsize = imsize
+        self.step_size = step_size
+
+    def __repr__(self) -> str:
+        return f"Unpatchify only supports reconstructing image with a uniform step size for all patches. \
+However, reconstructing {self.n_patches} x {self.patch_size}px patches to an {self.imsize} image requires {self.step_size} as step size, which is not an integer."
+
+    def __str__(self) -> str:
+        return self.__repr__()
